from typing import List

from pydantic import BaseModel, model_validator
from pydantic_core import PydanticUndefined


class ConfModel(BaseModel):
    @model_validator(mode="before")
    @classmethod
    def nested_defaults(cls, data):
        for name, field in cls.model_fields.items():
            expected_type = field.annotation
            if name not in data:
                if field.default is PydanticUndefined:
                    data[name] = expected_type
                else:
                    data[name] = field.default
        return data


class Total(ConfModel):
    """整体"""

    # 所在页面
    page: str = "init"
    # 是否已展示帮助文档
    is_already_show_doc: bool = False


class UpdatePart(ConfModel):
    """更新代码"""

    # mower-ng 代码分支
    branch: str = "slow"
    # PyPI 仓库镜像
    mirror: str = "aliyun"


class LaunchPart(ConfModel):
    """启动程序"""

    class Instance(ConfModel):
        """实例"""

        # 是否选中
        checked: bool = False
        # 实例名
        name: str = ""
        # 实例路径
        path: str = ""

    # 实例列表
    instances: List[Instance] = []
    # 是否展示日志窗口
    is_show_log: bool = False


class Conf(
    Total,
    UpdatePart,
    LaunchPart,
):
    pass
